//********************************************************************************************
// WS2811.c  RGB addressable LED Strip Clock and pattern display in an infinity mirror
// For PIC 18F2550
// Crystal 20 MHz  HSPLL 48Mhz
// Works with WS2811/WS2812B LEDs
// DS1307 real time clock running on I2C RB0 and RB1
// UM66 melody chip for hour notification
// LED Order is G, R, B.
// LED Strip - +5V, DAT_IN, GND   Needs about 3 Amps
// Pin 18, RC7,  Data Out to LED strip
// Pin 5, RA3, Clock/Patterns switch
// Pin 2, RA0 SET switch.             for setting time and date
// Pin 3, RA1 NEXT switch.            select date, month, year, hour, minute - shows a decimal point
// Pin 4, RA2 INCREMENT Switch.       increment selected
// Pin 23, RB2 SLEEP switch
// Pin 24, RB3 Melody out goes high for 26 seconds
// Pin 25, RB4 output to beeper, used on all switch presses
// WS2812B_LEDCount = 60
// Copyright Graham Payne commenced 22/2/2015
//*********************************************************************************************

// Pattern Routines
//*********************************************************************************************
//CometLeft(color)
//CometRight(color)
//RandomColor(number of times)                                random color at random position
//InsertColor(color)                                          inserts a color at position 0
//Fill_Dot(green value, red value, blue value, position)      position 0 to 59, color values 0 - 255
//MovLeft_Chase_and_Clear(position to 59)                     position 0 - 58
//MovRight_Chase_and_Clear(position to 0)                     position 1 - 59
//rainbowCycleRight(number of times)
//rainbowCycleLeft(number of times)
//FillLeft(color)
//FillRight(color)
//Solid_Strip(color)
//Fade_Strip(start color, end color, pos)                     if  pos = 0 do all leds else just the one in pos. pos up to 59
//Chase_3Color_Left(color, color, color, repeat times)        repeat times 1 to 255
//Chase_3Color_Right(color, color, color, repeat times)       repeat times 1 to 255
//Fill_Any_PosR(color, color, color, start pos, end pos)      start 0 - ?, end must be after start to 59
//Fill_Any_PosL(color, color, color, start pos, end pos)      start 59 - ?, end must be before start to 0
//Fill_Dot_PosR(color, start, end)                            counts forewards from start to end i.e 10 - 45
//Fill_Dot_PosL(color, start, end)                            counts backwards from start to end i.e 45 - 10
//ChaseRandom(number of times)
//Clear_Strip()                                               clears all to black

//****************************************************************************
//Port Declarations
//****************************************************************************

//output to melody chip
sbit Melody_Out at RB3_bit;
sbit Melody_Out_Direction at TRISB3_bit;

//output to strip
sbit LEDOut at RC7_bit;
sbit LEDOut_Direction at TRISC7_bit;
//switches
sbit SET_Switch at RA0_bit;
sbit NEXT_Switch at RA1_bit;
sbit INC_Switch at RA2_bit;
sbit SET_Switch_Direction at TRISA0_bit;
sbit NEXT_Switch_Direction at TRISA1_bit;
sbit INC_Switch_Direction at TRISA2_bit;

sbit Clock_Switch at RA3_bit;                    //clock/patterns switch
sbit Clock_Switch_Direction at TRISA3_bit;
sbit Sleep_Switch at RB2_bit;                    //sleep switch
sbit Sleep_Switch_Direction at TRISB2_bit;

// Software SPI module connections for MAX7912
sbit CS at RC0_bit;
sbit SoftSpi_SDO at RC1_bit;
sbit SoftSpi_CLK at RC2_bit;
sbit SoftSpi_SDI at RC6_bit;
sbit CS_direction at TRISC0_bit;
sbit SoftSpi_SDO_Direction at TRISC1_bit;
sbit SoftSpi_CLK_Direction at TRISC2_bit;
sbit SoftSpi_SDI_Direction at TRISC6_bit;

#define  testbit(var, bit)  ((var) & (1 <<(bit)))   //checks a nominated bit for a 1 or 0

#define        red            0,255,0               //colors, green, red, blue order
#define        green          255,0,0
#define        blue           0,0,255
#define        black          0,0,0
#define        grey           40,40,40
#define        white          255,255,255
#define        yellow         255,255,0
#define        pale_orange    20,50,0
#define        cyan           255,0,255
#define        magenta        0,255,255
#define        gold           160,255,0
#define        orange         20,255,0
#define        olive          220,100,47
#define        turq           245,40,100
#define        lime           254,64,20
#define        skyblue        206,135,255
#define        purple         0,75,130
#define        pink           30,255,12
#define        crimson        3,220,3

unsigned char sleep_mode = 0;
const unsigned char NumberOfLeds = 60;             // 60 LEDs
unsigned long led_strip[NumberOfLeds];             //holds 60 lots of 24 bits, 180 bytes, 1440 bits
unsigned char Strip[NumberOfLeds * 24];            //compiled data stream - 1440 bits
unsigned char second, minute, hour, day, day_number, date, month, year, r_data, Prev_hour, Prev_minute, Prev_second, Analog_hour, temp;

//debounce switches (library function) - &port, port number (0-7), debounce time (in mS), state 1 or 0
unsigned short Button(unsigned short *port, unsigned short pin, unsigned short debounce_time, unsigned short active_state);

void Beep()
{
  Sound_Play(300, 100);
}

//*******************************************************************************
// interrupt on external interrupt RB2 to activate/de-activate sleep
void interrupt() 
{
  if(INT2IF_bit == 1) 
  {
    INT2IF_bit = 0;              // clear the flag
    while(Sleep_Switch = 0);     //check for switch release
    delay_ms(600);
    sleep_mode = ~sleep_mode;    //toggle sleep mode
    if(!sleep_mode)              //if not sleep, wake up
    {
      asm reset;
      asm nop;
    }
  }
}
//*******************************************************************************
// Outputs a continuous string of 1's and 0's
void Output_Strip()
{
  unsigned int y;

  for (y = 0; y < 1440; y++)
   {
      if (Strip[y] == 1)    //output a one
       {
         LEDOut = 1;        //high for 800ns
         asm{
          nop
          nop
          nop
          nop
          nop
          nop
          nop
          }
       }
      else                  //output a zero
       {
          LEDOut = 1;       //high for 400ns
          asm{
          nop
          nop
          nop
          nop
          }
       }
      LEDOut = 0;           //low
    }
  delay_us(75);             //reset
}
//*******************************************************************************
//Main output routine, sends all bits to LEDS via RC7
void CompileLEDDataStream()                //outputs 3 bytes for each LED
{
    unsigned long this_led;
    unsigned char j, z;
    unsigned int s = 0;

    for (j = 0; j < NumberOfLeds; j++)    //iterate thru all Leds
    {
      this_led = led_strip[j];
      for (z = 0; z < 24; z++)            //3 bytes, 24 bits
      {
         if (testbit(this_led, 23)) {Strip[s] = 1;} else {Strip[s] = 0;}  //checks for a "1" at MSB bit 24 (23)  i.e. 10000000,00000000,00000000
         s++;                             // increment to next led
         this_led *= 2;                   //shift all bits left
      }
    }
   Output_Strip();
}

//****************************************************************************
//Sets the color of a SINGLE LED
void Set_Color(unsigned long g, unsigned int r, char b, char position)
{
   led_strip[position] = (g << 16) | (r <<  8) | b;   //set data in position nominated
}

//*******************************************************************************
//convert BCD from TRC to left most character i.e get the 2 from 25
unsigned char BCD2UpperCh(unsigned char bcd)
{
     return ((bcd >> 4)+ '0');
}

//*******************************************************************************
//convert BCD from TRC to right most character i.e get the 5 from 25
unsigned char BCD2LowerCh(unsigned char bcd)
{
     return ((bcd & 0x0f) + '0');
}

//*******************************************************************************
//Read fron the RTC
unsigned short read_ds1307(unsigned short address)
{
  I2C1_Start();
  I2C1_Wr(0xD0);               //address 0x68 followed by direction bit (0 for write, 1 for read) 0x68 followed by 0 --> 0xD0
  I2C1_Wr(address);
  I2C1_Repeated_Start();
  I2C1_Wr(0xD1);               //0x68 followed by 1 --> 0xD1
  r_data = I2C1_Rd(0);
  I2C1_Stop();
  return r_data;
}

//*******************************************************************************
//write to the RTC
void write_ds1307(unsigned short address,unsigned short w_data)
{
  I2C1_Start();                // issue I2C start signal
  I2C1_Wr(0xD0);               // send byte via I2C (device address + W)
  I2C1_Wr(address);            // send byte (address of DS1307 location)
  I2C1_Wr(w_data);             // send data (data to be written)
  I2C1_Stop();                 // issue I2C stop signal
}

//*******************************************************************************
//Get time and date
void Get_Time()
{
  second = read_ds1307(0);     // read second
  minute = read_ds1307(1);     // read minute
  hour = read_ds1307(2);       // read hour
  day = read_ds1307(3);        // read day
  date = read_ds1307(4);       // read date
  month = read_ds1307(5);      // read month
  year = read_ds1307(6);       // read year
}

//*******************************************************************************
//Intitial time and date setting. USE ONLY FOR INITIAL SETTING OF TIME
void Set_Initial_Time()
{
  write_ds1307(0,0x80);               //Reset second to 0 sec. and stop Oscillator
  write_ds1307(1,(Dec2Bcd(59)));      //write min
  write_ds1307(2,(Dec2Bcd(8)));      //write hour
  write_ds1307(3,(Dec2Bcd(4)));       //write day of week , 0 is sunday
  write_ds1307(4,(Dec2Bcd(21)));      //write date
  write_ds1307(5,(Dec2Bcd(5)));       //write month
  write_ds1307(6,(Dec2Bcd(15)));      //write year, last 2 digits only
  write_ds1307(7,0x00);               //TURN OFF SQWE output
  write_ds1307(0,0x00);               //0x00 Reset second to 0 sec. and start Oscillator
}

//*******************************************************************************
//Send character to the MAX7219
void send_to_max7219(char address, char dat)
{
  CS = 0;                             // LOAD(CS) LINE LOW
  Soft_SPI_Write(address);
  Soft_SPI_Write(dat);
  CS = 1;                             // LOAD(CS) LINE HIGH
}

//*******************************************************************************
//initiallise the MAX7219
void init_MAX7219() {
  send_to_max7219(0x09,0xFF);         // BCD Mode Code B
  send_to_max7219(0x0A,0x09);         // INTENSITY 9/15 th's
  send_to_max7219(0x0B,0x07);         // SCAN ALL DIGITS
  send_to_max7219(0x0C,0x01);         // TURN ON
  send_to_max7219(0x00,0x0F);         // TEST OFF
}

//*******************************************************************************
//Adjust time and date
void Adjust_Time_Date() 
{
   unsigned char set_count;
   if (Button(&PORTA, 0, 20, 0))                      //(SET_Switch == 0) if set switch pressed
   {
      Beep();
      delay_ms(100);
      set_count = 1;                                  //set next switch counter
      send_to_max7219(0x01, BCD2UpperCh(hour));       //display the hour and minute digits
      send_to_max7219(0x02, BCD2LowerCh(hour)+128);   //decimal point to show what we are adjusting
      send_to_max7219(0x03, BCD2UpperCh(minute));
      send_to_max7219(0x04, BCD2LowerCh(minute));
      send_to_max7219(0x05, 0);                       // seconda are zero
      send_to_max7219(0x06, 0);
      send_to_max7219(0x07, 15);                      //blank the last 2
      send_to_max7219(0x08, 15);

      while (Button(&PORTA, 0, 20, 0))                //(SET_Switch == 0) while set switch is pressed
      {
         while (Button(&PORTA, 1, 20, 0))             //(NEXT_Switch == 0) when next switch is pressed
         {
            delay_ms(425);
            set_count++;                              //increment
            Beep();
            if(set_count > 5) set_count = 1;          //5 only required, go back to beginning

             switch (set_count)                       //selecting which item we want to adjust
             {                                        //can come back to this as set_count doesnt change when adjusting
                 case 1:
                      send_to_max7219(0x01, BCD2UpperCh(hour));      // SET HOURS display the hour and minute digits
                      send_to_max7219(0x02, BCD2LowerCh(hour)+128);  //get to 2nd hour digit, display point
                      send_to_max7219(0x03, BCD2UpperCh(minute));
                      send_to_max7219(0x04, BCD2LowerCh(minute));
                      send_to_max7219(0x05, 0);
                      send_to_max7219(0x06, 0);
                      send_to_max7219(0x07, 15);                     // seconda are zero
                      send_to_max7219(0x08, 15);
                      break;
                 case 2:
                      send_to_max7219(0x01, BCD2UpperCh(hour));      //SET MINUTES display the hour and minute digits
                      send_to_max7219(0x02, BCD2LowerCh(hour));
                      send_to_max7219(0x03, BCD2UpperCh(minute));
                      send_to_max7219(0x04, BCD2LowerCh(minute)+128);//get to 2nd minute digit, display point
                      send_to_max7219(0x05, 0);
                      send_to_max7219(0x06, 0);
                      send_to_max7219(0x07, 15);                     // seconda are zero
                      send_to_max7219(0x08, 15);
                      break;                                         //dont bother changing seconds
                 case 3:
                      send_to_max7219(0x01, BCD2UpperCh(date));      //SET MONTH display date, month and year digits
                      send_to_max7219(0x02, BCD2LowerCh(date));
                      send_to_max7219(0x03, BCD2UpperCh(month));
                      send_to_max7219(0x04, BCD2LowerCh(month)+128); //display point on month
                      send_to_max7219(0x05, 2);
                      send_to_max7219(0x06, 0);
                      send_to_max7219(0x07, BCD2UpperCh(year));
                      send_to_max7219(0x08, BCD2LowerCh(year));
                      break;
                 case 4:
                      send_to_max7219(0x01, BCD2UpperCh(date));      //SET DATE display date, month and year digits
                      send_to_max7219(0x02, BCD2LowerCh(date)+128);  //display point on date
                      send_to_max7219(0x03, BCD2UpperCh(month));
                      send_to_max7219(0x04, BCD2LowerCh(month));
                      send_to_max7219(0x05, 2);
                      send_to_max7219(0x06, 0);
                      send_to_max7219(0x07, BCD2UpperCh(year));
                      send_to_max7219(0x08, BCD2LowerCh(year));
                      break;
                 case 5:
                      send_to_max7219(0x01, BCD2UpperCh(date));      //SET DATE display date, month and year digits
                      send_to_max7219(0x02, BCD2LowerCh(date));
                      send_to_max7219(0x03, BCD2UpperCh(month));
                      send_to_max7219(0x04, BCD2LowerCh(month));
                      send_to_max7219(0x05, 2);
                      send_to_max7219(0x06, 0);
                      send_to_max7219(0x07, BCD2UpperCh(year));
                      send_to_max7219(0x08, BCD2LowerCh(year)+128);  //display point on year
                      break;
              }                                                      //end switch select which digits
          }                                                          //end while NEXT pressed

          while (Button(&PORTA, 2, 20, 0))                           //(UP_Switch == 0) Adjust selected digit
          {
              delay_ms(425);
              Beep();
              switch (set_count)                                     //select the area we want to change
              {
               case 1:                                               //change hours
                   hour++;                                           //increment on first press of up switch
                   write_ds1307(0,0x80);                             //Reset second to 0 sec. and stop Oscillator
                   hour = BCD2dec(hour);                             //convert to something we can use
                   if(hour > 23) hour = 0;                           //dont go over 23 hours
                   write_ds1307(2,(dec2Bcd(hour)));                  //write hour
                   write_ds1307(0,0x00);                             //0x00 Reset second to 0 sec. and start Oscillator
                   hour = read_ds1307(2);                            // read hour
                   send_to_max7219(0x01, BCD2UpperCh(hour));         // display the digits
                   send_to_max7219(0x02, BCD2LowerCh(hour)+128);
                   break;
                case 2:                                              //change minutes
                   minute++;
                   write_ds1307(0,0x80);
                   minute = BCD2dec(minute);
                   if(minute > 59) minute = 0;
                   write_ds1307(1, (Dec2Bcd(minute)));
                   write_ds1307(0,0x00);
                   minute = read_ds1307(1);
                   send_to_max7219(0x03, BCD2UpperCh(minute));       // display the digits
                   send_to_max7219(0x04, BCD2LowerCh(minute)+128);
                 break;
                case 3:                                              //change month
                   month++;
                   write_ds1307(0,0x80);
                   month = BCD2dec(month);
                   if(month > 12) month = 1;
                   write_ds1307(5,(Dec2Bcd(month)));
                   write_ds1307(0,0x00);
                   month = read_ds1307(5);
                   send_to_max7219(0x03, BCD2UpperCh(month));        // display the digits
                   send_to_max7219(0x04, BCD2LowerCh(month)+128);
                  break;
                 case 4:                                             //change date
                   date++;
                   write_ds1307(0,0x80);
                   date = BCD2dec(date);

                   switch(month)                                     //check date against month
                   {
                      case 1: if(date > 31) date = 1; break;         //Jan
                      case 2:                                        //Feb, take care of leep years
                          if((date > 28) && (year % 4 != 0)) date = 1;
                          if(date > 29 && year % 4 == 0) date = 1;
                          break;
                      case 3: if(date > 31) date = 1; break;         //Mar
                      case 4: if(date > 30) date = 1; break;         //Apr
                      case 5: if(date > 31) date = 1; break;         //May
                      case 6: if(date > 30) date = 1; break;         //Jun
                      case 7: if(date > 31) date = 1; break;         //julif(month > 12) month = 1;
                      case 9: if(date > 30) date = 1; break;         //Sep
                      case 10: if(date > 31) date = 1; break;        //Oct
                      case 11: if(date > 30) date = 1; break;        //Nov
                      case 12: if(date > 31) date = 1; break;        //Dec
                   }
                   
                   write_ds1307(4, (Dec2Bcd(date)));
                   write_ds1307(0,0x00);
                   date = read_ds1307(4);
                   send_to_max7219(0x01, BCD2UpperCh(date));         // display the digits
                   send_to_max7219(0x02, BCD2LowerCh(date)+128);
                  break;
               case 5:                                               //change year
                   year++;
                   write_ds1307(0,0x80);
                   year = BCD2dec(year);
                   if(year >= 100) year = 0;
                   write_ds1307(6, (Dec2Bcd(year)));
                   write_ds1307(0,0x00);
                   year = read_ds1307(6);
                   send_to_max7219(0x07, BCD2UpperCh(year));         // display the digits
                   send_to_max7219(0x08, BCD2LowerCh(year)+128);
                  break;
              }                                                      //end switch set_count
          }                                                          //end while UP pressed
       }                                                             //end while SET pressed
    }                                                                //end if SET pressed - jump out now
}
//****************************************************************************
// Clear Strip
void Clear_Strip()
{
  unsigned char pos;
  for(pos = 0; pos < NumberOfLeds; pos++)
     {
     led_strip[pos] = 0x000000;
     }
  CompileLEDDataStream();
}

//*********************************************************************************
// Convert separate G,R,B into packed 32-bit RGB color.
unsigned long Strip_Color(char g, char r, char b)
{
   return ((unsigned long)g << 16) | ((unsigned long)r <<  8) | b;
}

//*********************************************************************************
// Input a value 0 to 255 to get a color value.
// The colours are a transition g - r - b - back to g.
unsigned long Wheel(unsigned char WheelPos)
{
  if(WheelPos < 85)
    {
      return Strip_Color(WheelPos * 3, 255 - WheelPos * 3, 0);
    }
  else if(WheelPos < 170)
    {
      WheelPos -= 85;
      return Strip_Color(255 - WheelPos * 3, 0, WheelPos * 3);
    }
  else
    {
      WheelPos -= 170;
      return Strip_Color(0, WheelPos * 3, 255 - WheelPos * 3);
    }
}

//****************************************************************************
void RandomColor(unsigned int loops)
{
  unsigned int a, b, c, d, j;
  for (j = 0; j < loops; j++)
    {
      srand(j);
      a = rand() % 255;
      b = rand() % 255;
      c = rand() % 255;
      d = rand() % NumberOfLeds;
      Set_Color(a, b, c, d);
      CompileLEDDataStream();
    }
}

//****************************************************************************
void ChaseRandom(unsigned int loops)
{
  unsigned int x, j;
  unsigned long new_color = 0;
  for (j = 0; j < loops; j++)
  {
    for(x = (NumberOfLeds - 1); x > 0; x--)     //First, shuffle all the current colors down one spot on the strip
      {
         led_strip[x] = led_strip[x - 1];
      }
    for(x = 0; x < 3; x++)                      //Now generate a new RGB color
      {
         new_color <<= 8;
         new_color |= rand();
      }
    led_strip[0] = new_color;                  //Add the new random color to the strip
    CompileLEDDataStream();
   }
}

//****************************************************************************
void Fade_Strip(unsigned char greennext, unsigned char rednext, unsigned char bluenext, unsigned char greenlast, unsigned char redlast, unsigned char bluelast, unsigned char pos)
{
  float rlast, glast, blast, rcurr, gcurr, bcurr, rcalc, gcalc, bcalc;    //Declare Temp Variables
  float MaxTemp, MaxStep, rtemp, gtemp, btemp;
  unsigned char b, nnn, count;
  unsigned long g, temp, temp1;
  unsigned int r;

  count = 0;
  gcurr =  greenlast;
  rcurr =  redlast;
  bcurr =  bluelast;
  glast =  greennext;
  rlast =  rednext;
  blast =  bluenext;
  MaxTemp = max((abs(gcurr - glast)), (abs(rcurr - rlast)));
  MaxStep = max((abs(bcurr - blast)), MaxTemp);
  gcalc = glast;
  rcalc = rlast;
  bcalc = blast;

  for (nnn = 0; nnn < MaxStep; nnn++)   // Calculate proportional fade rates
  {
    gtemp = (gcurr - glast) / MaxStep;
    rtemp = (rcurr - rlast) / MaxStep;
    btemp = (bcurr - blast) / MaxStep;
    gcalc = gcalc + gtemp;
    rcalc = rcalc + rtemp;
    bcalc = bcalc + btemp;
    g = gcalc;
    r = rcalc;
    b = bcalc;

    temp = 0;                          // Construct 24bit word
    temp1 = 0;
    temp1 = g << 16;
    temp = temp + temp1;
    temp1 = 0;
    temp1 = r << 8;
    temp = temp + temp1;
    temp1 = 0;
    temp1 = b;
    temp = temp + temp1;

    if(pos == 0)                       // Populate Faded Colors
    {
      for (count = 0; count <= NumberOfLeds; ++count)
        {
        led_strip[count] = temp;
        }
    }
    else
    {
      led_strip[pos] = temp;
    }
    Set_Color(g, r, b, pos);
    delay_ms(50);
    CompileLEDDataStream();            // Delay and Display Frame
  }
}

//****************************************************************************
void Solid_Strip(unsigned char g, unsigned char r, unsigned char b)
{
  unsigned char x;
  for(x = 0; x < NumberOfLeds; x ++)
   {
     Set_Color(g, r, b, x);
   }
  CompileLEDDataStream();
}

//****************************************************************************
void InsertColor(unsigned char g, unsigned char r, unsigned char b)
{
  unsigned int x;
  for(x = (NumberOfLeds - 1); x > 0; x--)
    {
      led_strip[x] = led_strip[x - 1];
    }
  Set_Color(g, r, b, 0);
  CompileLEDDataStream();
}

//****************************************************************************
void FillLeft(unsigned char g, unsigned char r, unsigned char b)
{
  unsigned char count;
  for(count = NumberOfLeds; count > 0; count--)
    {
      Set_Color(g, r, b, count - 1);
      CompileLEDDataStream();
    }
}

//****************************************************************************
void FillRight(unsigned char g, unsigned char r, unsigned char b)
{
  unsigned char count;
  for(count = 0; count < NumberOfLeds; count++)
    {
      Set_Color(g, r, b, count);
      CompileLEDDataStream();
    }
}

//****************************************************************************
//  Fill Right Any Position
void Fill_Any_PosR(unsigned char g1, unsigned char r1, unsigned char b1, unsigned char g2, unsigned char r2, unsigned char b2, unsigned char g3, unsigned char r3, unsigned char b3, unsigned char start, unsigned char end)
{
   unsigned char pos;
   for(pos = start; pos < end; pos++)
    {
      if((pos%3) == 0) {Set_Color(g3, r3, b3, pos);}
      if((pos%3) == 1) {Set_Color(g2, r2, b2, pos);}
      if((pos%3) == 2) {Set_Color(g1, r1, b1, pos);}

      CompileLEDDataStream();
    }
}

//****************************************************************************
//  Fill Left Any Position
void Fill_Any_PosL(unsigned char g1, unsigned char r1, unsigned char b1, unsigned char g2, unsigned char r2, unsigned char b2, unsigned char g3, unsigned char r3, unsigned char b3, unsigned char start, unsigned char end)
{
   unsigned char pos;
   for(pos = start; pos > end; pos--)
    {
      if((pos%3) == 0) {Set_Color(g1, r1, b1, pos - 1);}
      if((pos%3) == 1) {Set_Color(g2, r2, b2, pos - 1);}
      if((pos%3) == 2) {Set_Color(g3, r3, b3, pos - 1);}

      CompileLEDDataStream();
    }
}

//****************************************************************************
void Chase_3Color_Left(unsigned char g1, unsigned char r1, unsigned char b1, unsigned char g2, unsigned char r2, unsigned char b2, unsigned char g3, unsigned char r3, unsigned char b3, unsigned char repeat)
{
  unsigned char pos, loopframe;
  int strt;
  pos = NumberOfLeds;
  for(loopframe = 0; loopframe < repeat; loopframe++)   // Increments Led Position every ++pos
   {
     delay_ms(250);
     Set_Color(g3, r3, b3, pos + 2);
     Set_Color(g2, r2, b2, pos + 1);
     Set_Color(g1, r1, b1, pos);
     pos--;
     if (pos == NumberOfLeds - 3)
      {
        pos = NumberOfLeds;
      }
     for(strt = 0; strt < NumberOfLeds; strt++)
      {
        led_strip[strt] = led_strip[strt + 1];
      }
     CompileLEDDataStream();
   }
}

//****************************************************************************
void Chase_3Color_Right(unsigned char g1, unsigned char r1, unsigned char b1, unsigned char g2, unsigned char r2, unsigned char b2, unsigned char g3, unsigned char r3, unsigned char b3, unsigned char repeat)
{
  unsigned char loopframe, pos;
  int strt;
  pos = 0;
  for(loopframe = 0; loopframe < repeat; loopframe++)   // Increments loopframe NumberOfLeds counts including zero
     {
     delay_ms(250);
     Set_Color(g1, r1, b1, pos - 0);      // Increments Led Position every ++pos
     Set_Color(g2, r2, b2, pos - 1);
     Set_Color(g3, r3, b3, pos - 2);
     pos++;
     if(pos == 3) pos = 0;

     for(strt = NumberOfLeds; strt >= 0; strt--)   // Counts down from NumberOfLeds to 0
        {
          led_strip[strt + 1] = led_strip[strt];
        }
     CompileLEDDataStream();
     }
}

//****************************************************************************
void MovLeft_Chase_and_Clear(unsigned char position)
{
  unsigned char pos, loopframe;
  for(loopframe = 0; loopframe < position; loopframe++)
   {
   for(pos = 0; pos < NumberOfLeds; pos++)
      {
        led_strip[pos] = led_strip[pos + 1];
      }
   CompileLEDDataStream();
   delay_ms(100);
   led_strip[NumberOfLeds] = 0x000000;
   }
}

//****************************************************************************
void MovRight_Chase_and_Clear(unsigned char position)
{
  unsigned char pos, loopframe;
  for(loopframe = 0; loopframe < position; loopframe++)
   {
   for(pos = (NumberOfLeds - 1); pos > 0; pos--)
      {
        led_strip[pos] = led_strip[pos - 1];
      }
   CompileLEDDataStream();
   delay_ms(100);
   led_strip[0] = 0x000000;
   }
}

//****************************************************************************
//Sets a single LED at a predefined color and position
void Fill_Dot (unsigned char g, unsigned char r, unsigned char b, unsigned char pos)
{
  Set_Color(g, r, b, pos);            //change the color
  CompileLEDDataStream();             //refill LED strip
}

//****************************************************************************
void Fill_Dot_PosR (unsigned char g, unsigned char r, unsigned char b, unsigned char start, unsigned char end)
{
   unsigned char pos;
   for(pos = start; pos <= end; pos++)
    {
      delay_ms(90);
      Set_Color(g, r, b, pos);
      CompileLEDDataStream();
    }
}

//****************************************************************************
void Fill_Dot_PosL(unsigned char g, unsigned char r, unsigned char b, unsigned char start, unsigned char end)
{
   unsigned char pos;
   for(pos = start; pos >= end; pos--)
    {
      delay_ms(90);
      Set_Color(g, r, b, pos - 1);
    }
    CompileLEDDataStream();
}

//*********************************************************************************
// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycleLeft(unsigned char cycles)
{
  unsigned int j, i;
  for(j = 0; j < 256 * cycles; j++)
  {
    for(i = 0; i < NumberOfLeds; i++)
      {
        led_strip[i] = Wheel(((i * 256 / NumberOfLeds) + j) & 255);
      }
    CompileLEDDataStream();
  }
}

//*********************************************************************************
// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycleRight(unsigned char cycles)
{
  unsigned int j, i;
  for(j = 256 * cycles; j--;)
  {
    for(i = 0; i < NumberOfLeds; i++)
      {
        led_strip[i] = Wheel(((i * 256 / NumberOfLeds) + j) & 255);
      }
    CompileLEDDataStream();
  }
}

//***************************************************************************
void CometLeft(unsigned char g, unsigned char r, unsigned char b)
{
  unsigned char v;
  for(v = 0; v < NumberOfLeds; v++)
    {
      delay_ms(60);
      Set_Color(g, r, b, v);
      Set_Color(g / 2, r / 2, b / 2, v - 1);
      Set_Color(g / 4, r / 4, b / 4, v - 2);
      Set_Color(g / 8, r / 8, b / 8, v - 3);
      Set_Color(g / 16, r / 16, b / 16, v - 4);
      Set_Color(g / 32, r / 32, b / 32, v - 5);
      Set_Color(g / 64, r / 64, b / 64, v - 6);
      Set_Color(g / 128, r / 128, b / 128, v - 7);
      Set_Color(0, 0, 0, v - 8);
      CompileLEDDataStream();
    }
  Clear_Strip();
}

//*****************************************************************************
void CometRight(unsigned char g, unsigned char r, unsigned char b)
{
  unsigned char v;
  for(v = NumberOfLeds; v--;)
    {
      delay_ms(60);
      Set_Color(g, r, b, v - 9);
      Set_Color(g / 2, r / 2, b / 2, v - 8);
      Set_Color(g / 4, r / 4, b / 4, v - 7);
      Set_Color(g / 8, r / 8, b / 8, v - 6);
      Set_Color(g / 16, r / 16, b / 16, v - 5);
      Set_Color(g / 32, r / 32, b / 32, v - 4);
      Set_Color(g / 64, r / 64, b / 64, v - 3);
      Set_Color(g / 128, r / 128, b / 128, v - 2 );
      Set_Color(0, 0, 0, v - 1);
      CompileLEDDataStream();
    }
}

//*******************************************************************************
// sends blanking commands to all digits
void Blank_MAX7219()
{
   send_to_max7219(0x01, 15);         //blank the LEDs
   send_to_max7219(0x02, 15);
   send_to_max7219(0x03, 15);
   send_to_max7219(0x04, 15);
   send_to_max7219(0x05, 15);
   send_to_max7219(0x06, 15);
   send_to_max7219(0x07, 15);
   send_to_max7219(0x08, 15);
}

//*******************************************************************************
// runs the Pattern sequence
void Run_Patterns()
{
   unsigned char counter;
   Beep();
   if(Melody_Out == 1) {Melody_Out = 0;}          //make sure melody not playing else it wont stop
   Blank_MAX7219();                               //blank off the date display
   Clear_Strip();                                 //clear all leds
   
   //start patterns here ****************************************************
   RandomColor(100);

   CometRight(red);
   CometLeft(green);
   CometRight(blue);
   CometLeft(yellow);

   rainbowCycleRight(2);
   rainbowCycleLeft(2);
   delay_ms(500);

   Clear_Strip();
   RandomColor(500);
   delay_ms(500);

   for(counter = 0; counter < 10; ++counter)
   {
     InsertColor(black);
     delay_ms(90);
     InsertColor(turq);
      delay_ms(90);
     InsertColor(magenta);
      delay_ms(90);
     InsertColor(yellow);
      delay_ms(90);
     InsertColor(pink);
      delay_ms(90);
     InsertColor(red);
      delay_ms(90);
     InsertColor(white);
      delay_ms(90);
     InsertColor(crimson);
      delay_ms(90);
     InsertColor(green);
      delay_ms(90);
     InsertColor(orange);
      delay_ms(90);
     InsertColor(blue);
      delay_ms(90);
     InsertColor(olive);
      delay_ms(90);
     InsertColor(purple);
      delay_ms(90);
     InsertColor(lime);
      delay_ms(90);
     InsertColor(skyblue);
      delay_ms(90);
     InsertColor(gold);
      delay_ms(90);
   }
   Clear_Strip();

   Fill_Dot_PosR(red, 0, 59);
   delay_ms(100);
   Fill_Dot_PosR(green, 17, 45);
   delay_ms(100);
   Fill_Dot_PosR(blue, 25, 35);

   delay_ms(500);
   Solid_Strip(yellow);
   delay_ms(500);
   Solid_Strip(gold);
   delay_ms(500);
   Solid_Strip(pink);
   delay_ms(500);
   Solid_Strip(lime);
   delay_ms(500);
   Solid_Strip(red);
   delay_ms(500);

   MovLeft_Chase_and_Clear(NumberOfLeds);
   delay_ms(500);
   MovRight_Chase_and_Clear(NumberOfLeds);
   delay_ms(500);

   Solid_Strip(turq);
   delay_ms(500);
   ChaseRandom(1000);
   RandomColor(1000);

   rainbowCycleRight(2);
   delay_ms(500);
   rainbowCycleLeft(2);
   delay_ms(500);

   FillLeft(red);
   delay_ms(200);
   FillRight(black);
   delay_ms(200);
   FillRight(purple);
   delay_ms(200);
   FillRight(black);
   delay_ms(200);
   FillLeft(crimson);
   delay_ms(200);
   FillLeft(black);
   delay_ms(200);
   FillRight(orange);
   delay_ms(200);
   FillRight(black);
   delay_ms(200);
   FillLeft(lime);
   delay_ms(200);
   FillLeft(black);
   delay_ms(200);
   FillRight(magenta);
   delay_ms(200);
   FillLeft(black);
   delay_ms(200);
   FillLeft(cyan);
   delay_ms(200);
   FillRight(black);
   delay_ms(200);
   FillRight(olive);
   FillRight(black);
   delay_ms(200);

   Chase_3Color_Right(cyan, orange, lime, 59);
   delay_ms(500);
   Fade_Strip(black, orange, 0);
   Fade_Strip(orange, blue, 0);
   Fade_Strip(blue, red, 0);
   Fade_Strip(red, green, 0);
   Fade_Strip(green, purple, 0);
   delay_ms(500);

   Chase_3Color_Right(green, red, blue, 59);
   Chase_3Color_Left(yellow, cyan, red, 59);
   Chase_3Color_Right(crimson, skyblue, green, 59);
   Chase_3Color_Left(blue, purple, yellow, 59);
   Chase_3Color_Right(magenta, olive, lime, 59);
   Chase_3Color_Left(gold, pink, gold, 59);
   Chase_3Color_Right(purple, turq, red, 59);
   Chase_3Color_Left(lime, blue, grey, 59);
   Chase_3Color_Right(yellow, white, crimson, 59);
   Chase_3Color_Left(skyblue, pink, cyan, 59);

   delay_ms(500);
   Fill_Any_PosR(red, blue, green, 0, NumberOfLeds);
   delay_ms(500);
   MovLeft_Chase_and_Clear(NumberOfLeds);
   delay_ms(500);
   Fill_Any_PosR(red, green, yellow, 0, NumberOfLeds);
   delay_ms(500);
   MovRight_Chase_and_Clear(NumberOfLeds);
   delay_ms(500);
   FillLeft(purple);
   delay_ms(500);
   Fill_Any_PosR(magenta, black, black, 0, NumberOfLeds);
   delay_ms(500);
   MovRight_Chase_and_Clear(NumberOfLeds);
   delay_ms(500);
   Fill_Any_PosR(gold, gold, gold, 0, NumberOfLeds);
   delay_ms(500);
   MovLeft_Chase_and_Clear(NumberOfLeds);
   delay_ms(500);
   FillRight(skyblue);
   delay_ms(500);

   CometRight(red);
   CometLeft(green);
   CometRight(blue);
   CometLeft(yellow);

   rainbowCycleRight(2);
   rainbowCycleLeft(2);
   
   RandomColor(1000);
   
   Fill_Any_PosL(blue, yellow, yellow, NumberOfLeds - 1, 0);
   delay_ms(500);

   Fill_Any_PosR(magenta, black, black, 0, NumberOfLeds);
   delay_ms(500);

   Fill_Any_PosL(lime, orange, pink, NumberOfLeds - 1, 0);
   delay_ms(500);

   Fill_Any_PosR(magenta, grey, purple, 0, NumberOfLeds);
   delay_ms(500);

   Fill_Any_PosL(magenta, blue, yellow, NumberOfLeds - 1, 0);
   delay_ms(500);

   Fill_Any_PosR(pink, olive, crimson, 0, NumberOfLeds);
   delay_ms(500);

   Fill_Any_PosL(grey, purple, grey, NumberOfLeds - 1, 0);
   delay_ms(500);

   Fill_Any_PosR(blue, orange, lime, 0, NumberOfLeds);
   delay_ms(500);

   Fill_Any_PosL(gold, cyan, crimson, NumberOfLeds - 1, 0);
   delay_ms(500);

   Fill_Any_PosR(gold, turq, skyblue, 0, NumberOfLeds);
   delay_ms(500);

   Fill_Any_PosL(red, green, red, NumberOfLeds - 1, 0);
   delay_ms(500);

   Fill_Any_PosR(red, blue, black, 0, NumberOfLeds);
   delay_ms(1000);

   Fill_Any_PosL(red, black, black, NumberOfLeds - 1, 0);
   delay_ms(1000);

   Clear_Strip();                               //finished
                                                //get the time back as the clock stops while patterns are running
   minute = BCD2dec(read_ds1307(1));            //read minutes
   delay_us(500);
   hour = BCD2dec(read_ds1307(2));              //read hours
   delay_us(500);
   temp = read_ds1307(0);                       //seconds
   delay_us(500);
   temp &= 0x7F;                                //add bit 7 to start osc
   
   write_ds1307(0,0x80);                        //Reset second to 0 sec. and stop Oscillator
   delay_us(500);
   write_ds1307(1,(Dec2Bcd(Minute)));           //write min
   delay_us(500);
   write_ds1307(2,(Dec2Bcd(Hour)));             //write hour
   delay_us(500);
   write_ds1307(0, temp);                       //write seconds and start clock
   delay_us(500);
}

//*******************************************************************************
// Run clock sequence
void Run_Clock()
{
  unsigned char i, hour_pos;

  Get_Time();                                        //get the current time and date

  Adjust_Time_Date();                                //for setting the time and date only if SET switch pressed

  send_to_max7219(0x01, BCD2UpperCh(date));          // display the date on 7 seg display
  send_to_max7219(0x02, BCD2LowerCh(date));
  send_to_max7219(0x03, BCD2UpperCh(month));
  send_to_max7219(0x04, BCD2LowerCh(month));
  send_to_max7219(0x05, 2);
  send_to_max7219(0x06, 0);
  send_to_max7219(0x07, BCD2UpperCh(year));
  send_to_max7219(0x08, BCD2LowerCh(year));

  led_strip[Prev_hour] = 0x000000;                   //clear the clock of old hour, minute and second
  led_strip[Prev_minute] = 0x000000;
  led_strip[Prev_second] = 0x000000;

  Analog_hour = BCD2dec(hour);                       //convert to something we can use
  minute = BCD2dec(minute);
  second = BCD2dec(second);

  if(Analog_hour > 11){Analog_hour -= 12;}           //set hour to 12 hour system, 0 - 11 for analog clock
  
  for(i = 0; i < NumberOfLeds; i ++)                 //place hour markers
  {
     if (i % 5 == 0){Set_Color(pale_orange, i);}
  }
  
  if(minute == 0 && second == 0){Melody_Out = 1;}   // on the hour, start the melody
  if(minute == 0 && second == 26){delay_ms(500); Melody_Out = 0;}    // turn off melody chip
  
  hour_pos = (Analog_hour * 5) + (minute / 12);           //get to correct LED position, 1 o'clock is at 5 min mark
                                                          //and move the hour hand every 12 minutes to simulate analog clock
  Set_Color(blue, hour_pos);                              //set the pixel for hour
  Set_Color(green, minute);                               //place minute position
  Set_Color(red, second);                                 //place second position
  
  if (minute == hour_pos){Set_Color(magenta, hour_pos);}  //check for overlapping hour, minute and seconds, change color accordingly
  if (second == hour_pos){Set_Color(blue, hour_pos);}     //dont show seconds over hour
  if (second == minute){Set_Color(green, minute);}        //dont show seconds over minute
  
  Prev_hour = hour_pos;                                   //save values for clearing next time
  Prev_minute = minute;
  Prev_second = second;
  
  CompileLEDDataStream();                                 //send to leds
}

//****************************************************************************
void main() {

  CMCON = 7;                              //turn off comparators
  ADCON0 = 0b00000000;                    //turn off A/D converters
  ADCON1 = 0b00001111;                    //all digital

  PORTA = 0;
  PORTB = 0;
  PORTC = 0;
  TRISA = 0b00001111;                     //RA0 Set sw, RA1 Next sw, RA2 Inc sw, RA3 clock/pattern switch,
  TRISB = 0b00000101;                     //RB2 is sleep switch on external INT2, RB0, RB1 for DS1307
  TRISC = 0b00000000;                     //RC7 is output to strip
  I2C1_Init(100000);                      //DS1307 I2C is running at 100KHz for RTC
  delay_ms(150);
  //Set_Initial_Time();                   //only do this once if battery connected to DS1307
  delay_ms(150);                          //then reprogram with this commented out
  Soft_SPI_Init();                        // Initialize Soft_SPI for MAX7912
  delay_ms(100);
  init_MAX7219();                         //init the MAX7219 chip
  delay_ms(100);

  Sound_Init(&PORTB, 4);                  //init sound for beeper
  
  GIE_bit = 1;                            // global interrupt enabled
  INTEDG2_bit = 0;                        // falling edge detection on INT2
  INT2IE_bit = 1;                         // interrupt enabled on INT2

  Clear_Strip();                          //turn off all leds
  
  Get_Time();                             //get the current time and date

  temp = read_ds1307(0);                  //read seconds
  delay_us(500);
  temp &= 0x7F;
  write_ds1307(0, temp);                  //write seconds and start osc
  delay_us(500);

 do
  {
    if(sleep_mode)             //if Sleep_mode = 1,  sleep
    {
        Blank_MAX7219();       //turn off  the 7 segs
        Clear_Strip();         //turn off all leds
        delay_ms(100);
        asm sleep;             //sleep
        asm nop;
     }

     if(Button(&PORTA, 3, 50, 0)) Run_Patterns(); // run the patterns routine

     Run_Clock();              //run the clock continuously

  }while(1);
}